package sun.java2d.pipe;

import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.BasicStroke;
import java.awt.Shape;
import sun.java2d.SunGraphics2D;

public class AAShapePipe implements ShapeDrawPipe, ParallelogramPipe
{
    static RenderingEngine renderengine;
    CompositePipe outpipe;
    private static byte[] theTile;
    
    public AAShapePipe(CompositePipe outpipe) {
        this.outpipe = outpipe;
    }
    
    @Override
    public void draw(SunGraphics2D sunGraphics2D, Shape strokedShape) {
        BasicStroke basicStroke;
        if ((sunGraphics2D.stroke instanceof BasicStroke)) {
            basicStroke = (BasicStroke)sunGraphics2D.stroke;
        }
        else {
            strokedShape = sunGraphics2D.stroke.createStrokedShape(strokedShape);
            basicStroke = null;
        }
        this.renderPath(sunGraphics2D, strokedShape, basicStroke);
    }
    
    @Override
    public void fill(SunGraphics2D sunGraphics2D, Shape shape) {
        this.renderPath(sunGraphics2D, shape, null);
    }
    
    private static Rectangle2D computeBBox(double x, double y, double w, double h) {
        if ((w -= x) < 0.0) {
            x += w;
            w = -w;
        }
        if ((h -= y) < 0.0) {
            y += h;
            h = -h;
        }
        return new Rectangle2D.Double(x, y, w, h);
    }
    
    @Override
    public void fillParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10) {
        Region compClip = sunGraphics2D.getCompClip();
        int[] array = new int[4];
        AATileGenerator aaTileGenerator = AAShapePipe.renderengine.getAATileGenerator(n5, n6, n7, n8, n9, n10, 0.0, 0.0, compClip, array);
        if (aaTileGenerator == null) {
            return;
        }
        this.renderTiles(sunGraphics2D, computeBBox(n, n2, n3, n4), aaTileGenerator, array);
        return;
    }
    
    @Override
    public void drawParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10, double n11, double n12) {
        Region compClip = sunGraphics2D.getCompClip();
        int[] array = new int[4];
        AATileGenerator aaTileGenerator = AAShapePipe.renderengine.getAATileGenerator(n5, n6, n7, n8, n9, n10, n11, n12, compClip, array);
        if (aaTileGenerator == null) {
            return;
        }
        this.renderTiles(sunGraphics2D, computeBBox(n, n2, n3, n4), aaTileGenerator, array);
        return;
    }
    
    private static synchronized byte[] getAlphaTile(int n) {
        byte[] theTile = AAShapePipe.theTile;
        if (theTile == null || theTile.length < n) {
            theTile = new byte[n];
            return theTile;
        }
        AAShapePipe.theTile = null;
        return theTile;
    }
    
    private static synchronized void dropAlphaTile(byte[] theTile) {
        AAShapePipe.theTile = theTile;
    }
    
    public void renderPath(SunGraphics2D sunGraphics2D, Shape shape, BasicStroke basicStroke) {
        int n = basicStroke != null && sunGraphics2D.strokeHint != 2;
        int n2 = sunGraphics2D.strokeState <= 1;
        Region compClip = sunGraphics2D.getCompClip();
        int[] array = new int[4];
        AATileGenerator aaTileGenerator = AAShapePipe.renderengine.getAATileGenerator(shape, sunGraphics2D.transform, compClip, basicStroke, (boolean)n2, (boolean)n, array);
        if (aaTileGenerator == null) {
            return;
        }
        this.renderTiles(sunGraphics2D, shape, aaTileGenerator, array);
        return;
    }
    
    public void renderTiles(SunGraphics2D sunGraphics2D, Shape shape, AATileGenerator aaTileGenerator, int[] array) {
        Object startSequence = null;
        byte[] alphaTile = null;
        try {
            startSequence = this.outpipe.startSequence(sunGraphics2D, shape, new Rectangle(array[0], array[1], array[2] - array[0], array[3] - array[1]), array);
            int tileWidth = aaTileGenerator.getTileWidth();
            int tileHeight = aaTileGenerator.getTileHeight();
            alphaTile = getAlphaTile(tileWidth * tileHeight);
            for (int i = array[1]; i < array[3]; i += tileHeight) {
                for (int j = array[0]; j < array[2]; j += tileWidth) {
                    int min = Math.min(tileWidth, array[2] - j);
                    int min2 = Math.min(tileHeight, array[3] - i);
                    int typicalAlpha = aaTileGenerator.getTypicalAlpha();
                    if (typicalAlpha == 0 || !this.outpipe.needTile(startSequence, j, i, min, min2)) {
                        aaTileGenerator.nextTile();
                        this.outpipe.skipTile(startSequence, j, i);
                    }
                    else {
                        byte[] array2;
                        if (typicalAlpha == 255) {
                            array2 = null;
                            aaTileGenerator.nextTile();
                        }
                        else {
                            array2 = alphaTile;
                            aaTileGenerator.getAlpha(alphaTile, 0, tileWidth);
                        }
                        this.outpipe.renderPathTile(startSequence, array2, 0, tileWidth, j, i, min, min2);
                    }
                }
            }
        }
        finally {
            aaTileGenerator.dispose();
            if (startSequence != null) {
                this.outpipe.endSequence(startSequence);
            }
            if (alphaTile != null) {
                dropAlphaTile(alphaTile);
            }
        }
    }
    
    static {
        AAShapePipe.renderengine = RenderingEngine.getInstance();
    }
}
